/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects;

import java.awt.Color;
import java.awt.Point;

/**
 * This interface represents GraphicalObject.
 *
 * Created 28.10.2009 at 13:10:27.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public interface GraphicalObject {

    /**
     * Indicates wheter the point is part of graphical object or not
     * 
     * @param point in question
     * @return true if given point is part of graphical object (either of line or graphical 
     * object's content), false else
     */
    boolean contains(Coordinate coordinate);

    /**
     * @return color of graphical object
     */
    Color getColor();

    /**
     *
     * @return current position of chosen point of graphical object. Chosen point remains the
     * same while this graphical objects exists.
     */
    Coordinate getPosition();

    /**
     * Changes graphical objects position relatively to current position.
     *
     * @param relX change of x coordinate
     * @param relY change of y coordinate
     * @return new position
     */
    Coordinate moveRelatively(int relX, int relY);

    /**
     * Changes graphical objects position absolutely to the given coortinate on screen
     *
     * @param toPosition new grafical object position
     */
    void moveAbsolutely(Coordinate toPosition);

}
