/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.viewer;

import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Circle;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Coordinate;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.GraphicalObject;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Line;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Rectangle;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Square;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Tetragon;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.Triangle;
import cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects.TwoDimensionalGraphicalObject;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.Timer;

/**
 * This class represents GraphicalObjectDisplayer.
 *
 * Created 29.10.2009 at 4:26:34.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class GraphicalObjectDisplayer {

    private JPanel canvas;
    private JTextField toStr = new JTextField("");
    private JTextField areaField = new JTextField("");
    private List<GraphicalObject> instaled = new ArrayList<GraphicalObject>();
    private Timer repaint;

    public GraphicalObjectDisplayer() {
        canvas = new GraficPanel();
        toStr.setEditable(false);
        areaField.setEditable(false);

//        canvas.setForeground(Color.WHITE);
        repaint = new Timer(5000000, new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                canvas.repaint();
            }
        });
    }

    public JTextField getAreaField() {
        return areaField;
    }

    public JTextField getToStr() {
        return toStr;
    }



    public JPanel getCanvas() {
        return canvas;
    }

    private class GraficPanel extends JPanel{
        
        private Point pressed;

        public GraficPanel() {
            super.addMouseListener(new MouseAdapter() {

                @Override
                public void mousePressed(MouseEvent e) {
                    pressed = e.getPoint();
                    showInfo(e);
                }

                @Override
                public void mouseReleased(MouseEvent e) {


                    if(pressed != null){
                        int moveX = (int) (e.getX() - pressed.getX());
                        int moveY = (int) (e.getY() - pressed.getY());
                        for(int i = instaled.size() - 1; i >= 0; i--){
                            if(instaled.get(i).contains(
                                    new Coordinate((int)pressed.getX(), (int)pressed.getY()))){
                                instaled.get(i).moveRelatively(moveX , moveY);
                                pressed = null;
                                showInfo(e);
                                repaint();
                                break;
                            }
                        }
                    }
                    showInfo(e);
                }

                private void showInfo(MouseEvent e) {
                    for (int i = instaled.size() - 1; i >= 0; i--) {
                        if (instaled.get(i).contains(new Coordinate(e.getX(), e.getY()))) {
                            toStr.setText(instaled.get(i).toString());
                            if (instaled.get(i) instanceof TwoDimensionalGraphicalObject) {
                                TwoDimensionalGraphicalObject tdgo = (TwoDimensionalGraphicalObject) instaled.get(i);
                                areaField.setText("" + tdgo.getArea());
                            } else {
                                areaField.setText("no area");
                            }
                            return;
                        }
                    }
                    areaField.setText("");
                    toStr.setText("");
                }

            });
        }

        @Override
        public synchronized void paint(Graphics g) {
            Graphics2D gra = (Graphics2D) g;
            int maxX = getSize().width;
            int maxY = getSize().height;
            boolean contained;
            for(int x = 0; x < maxX; x++){
                for(int y = 0; y < maxY; y++){
                    contained = false;
                    for(int i = instaled.size() - 1; i >= 0; i--){
                        if(instaled.get(i).contains(new Coordinate(x, y))){
                            gra.setColor(instaled.get(i).getColor());
                            gra.drawLine(x, y, x, y);
                            contained = true;
                            break;
                        }
                    }
                    if(!contained){
                        gra.setColor(Color.WHITE);
                        gra.drawLine(x, y, x, y);
                    }
                }
            }
        }


    }

    public void repaint(){
        canvas.repaint();
    }

    public synchronized void install(GraphicalObject newGO){
        instaled.add(newGO);
    }

}
